@@ -22,12 +22,13 @@
 //#endif
 #include "CdbPython.hh"
 #include "SympyCdb.hh"
-#include "pythoncdb/py_helpers.hh"
 
-// #define DEBUG 1
+//#define DEBUG 1
 
 
-bool interrupt_block=false;
+using websocketpp::lib::placeholders::_1;
+using websocketpp::lib::placeholders::_2;
+using websocketpp::lib::bind;
 
 Server::Server()
 //	: return_cell_id(std::numeric_limits<uint64_t>::max()/2)
@@ -35,12 +36,14 @@ Server::Server()
 	boost::uuids::uuid authentication_uuid = boost::uuids::random_generator()();
 	authentication_token = boost::uuids::to_string( authentication_uuid );
 	// FIXME: we do not actually do anything with this.
+	socket_name="tcp://localhost:5454";
 	init();
 	}
 
 Server::Server(const std::string& socket)
 //	: return_cell_id(std::numeric_limits<uint64_t>::max()/2)
 	{
+	socket_name=socket;
 	init();
 	}
 
@@ -141,12 +144,10 @@ void Server::init()
 
 	// Make the C++ CatchOutput class visible on the Python side.
 
-	auto python_path = cadabra::install_prefix_of_module();
-
-	// FIXME: since the logic above *requires* that we can find the
-	// `cdb.main` module, we will already have the correct path
-	// set. So appending it once more below is useless.
+	auto python_path = std::string(PYTHON_SITE_PATH);
 	
+	// auto python_path = std::string(cadabra::install_prefix() + "/lib/python" + std::to_string(PY_MAJOR_VERSION) + "." + std::to_string(PY_MINOR_VERSION) + "/" + std::string(PYTHON_SITE_DIST));
+
 	std::string stdOutErr =
 	   "import sys\n"
 	   "sys.path.append(r'"+python_path+"')\n"
@@ -175,22 +176,16 @@ void Server::init()
 		throw;
 		}
 
-	// Get the Python thread id.
-	std::string code_get_id = "import threading; print(threading.get_native_id())";
-	std::string main_thread_id_str = run_string(code_get_id);
-	main_thread_id = std::stol(main_thread_id_str);
-	// std::cerr << "Server: main python thread id = " << main_thread_id << std::endl;
-	// std::cerr << "Server: python_path = " << python_path << std::endl;
-	
+
 	// Call the Cadabra default initialisation script.
 
-	//	pybind11::eval_file(python_path + "/cadabra2_defaults.py");
+	//	pybind11::eval_file(PYTHON_SITE_PATH"/cadabra2_defaults.py");
 	//	HERE: should use pybind11::eval_file instead, much simpler.
 	//
 	std::string startup =
 	   "f=open(r'" + python_path + "/cadabra2_defaults.py'); "
 	   "code=compile(f.read(), 'cadabra2_defaults.py', 'exec'); "
-	   "exec(code); f.close() ";
+	   "exec(code); f.close()";
 	run_string(startup);
 
 #ifdef DEBUG
@@ -198,60 +193,28 @@ void Server::init()
 #endif
 	}
 
-int InterruptCheck(PyObject* obj, _frame* frame, int what, PyObject* arg)
-	{
-	std::cerr << "Server: interruptcheck" << std::endl;
-	if(interrupt_block) {
-		PyErr_SetString(PyExc_KeyboardInterrupt, "Stop script");
-		interrupt_block = false;
-		}
-	
-	return 0;
-	}
-
-std::string Server::run_string(const std::string& blk, bool handle_output, bool extract_variables,
-										 std::set<std::string> remove_assignments)
+std::string Server::run_string(const std::string& blk, bool handle_output)
 	{
 	//std::cerr << "RUN_STRING" << std::endl;
 	// snoop::log("run") << blk << snoop::flush;
 
-	std::string result, newblk;
-	
+	std::string result;
+
+	// Preparse input block.
+	auto newblk = cadabra::cdb2python_string(blk, true);
+
+	//	std::cerr << "PREPARSED:\n" << newblk << std::endl;
+	// snoop::log("preparsed") << newblk << snoop::flush;
+
 	// Run block. Catch output.
 	try {
-		// Preparse input block.
-		// std::cerr << "RAW:\n" << blk << std::endl;
-		std::string error;
-		newblk = cadabra::cdb2python_string(blk, true, error);
-		
-		// std::cerr << "PREPARSED:\n" << newblk << std::endl;
-		// snoop::log("preparsed") << newblk << snoop::flush;
-		
-		run_string_variables.clear();
-		if(error.size()==0) {
-			// If the preparsing found an error, do not attempt anything
-			// else; just run it and let Python report the error.
-			if(extract_variables) {
-				cadabra::variables_in_code(newblk, run_string_variables);
-				cadabra::variables_to_pull_in(newblk, run_string_variables);
-				// std::cerr << "----" << std::endl;
-				// for(const auto& name: run_string_variables)
-				// 	std::cerr << "contains " << name << std::endl;
-				}
-			for(const std::string& var: remove_assignments) {
-				newblk = cadabra::remove_variable_assignments(newblk, var);
-				}
-			// std::cerr << "REMOVED:\n" << newblk << std::endl;
-			}
-		
+		//		pybind11::object ignored = pybind11::eval<pybind11::eval_statements>(newblk.c_str(), main_namespace);
 #ifdef DEBUG
 		std::cerr << "executing..." << std::endl;
 		std::cerr << newblk << std::endl;
 #endif
 		PyErr_Clear();
-//		PyEval_SetTrace(InterruptCheck, NULL);		
 		pybind11::exec(newblk.c_str(), main_namespace);
-//		PyEval_SetTrace(NULL, NULL);		
 #ifdef DEBUG
 		std::cerr << "exec done" << std::endl;
 #endif
@@ -261,10 +224,6 @@ std::string Server::run_string(const std::string& blk, bool handle_output, bool
 		if(handle_output) {
 			result = catchOut.str();
 			catchOut.clear();
-			std::string result_err = catchErr.str();
-			if(result_err!="") 
-				std::cerr << "catchErr: " << result_err << std::endl;
-			catchErr.clear();
 			}
 		}
 	catch(pybind11::error_already_set& ex) {
@@ -280,12 +239,6 @@ std::string Server::run_string(const std::string& blk, bool handle_output, bool
 		// error come back on any future pybind11::exec() call.
 		std::string reason=parse_error(ex.what(), newblk);
 		ex.restore();
-		if(reason.substr(0, 17)=="KeyboardInterrupt") {
-			auto loc = reason.find("At:");
-			reason = "Interrupted a" + reason.substr(loc+1);
-			}
-//		std::cerr << "gobbling " << catchOut.str() << std::endl;
-//		catchOut.clear();
 		throw std::runtime_error(reason);
 		}
 
@@ -293,33 +246,33 @@ std::string Server::run_string(const std::string& blk, bool handle_output, bool
 	return result;
 	}
 
-//void Server::on_socket_init(websocketpp::connection_hdl, boost::asio::ip::tcp::socket & /* s */)
-//	{
-//	boost::asio::ip::tcp::no_delay option(true);
-//	// FIXME: this used to work in older websocketpp
-////	s.lowest_layer().set_option(option);
-//	}
+void Server::on_socket_init(websocketpp::connection_hdl, boost::asio::ip::tcp::socket & /* s */)
+	{
+	boost::asio::ip::tcp::no_delay option(true);
+	// FIXME: this used to work in older websocketpp
+//	s.lowest_layer().set_option(option);
+	}
 
 Server::Connection::Connection()
 	{
 	uuid = boost::uuids::random_generator()();
 	}
 
-void Server::on_open(websocket_server::id_type ws_id)
+void Server::on_open(websocketpp::connection_hdl hdl)
 	{
 	std::lock_guard<std::mutex> lock(ws_mutex);
 	Connection con;
-	con.ws_id = ws_id;
+	con.hdl=hdl;
 	// snoop::log(snoop::info) << "Connection " << con.uuid << " open." << snoop::flush;
-	connections[ws_id]=con;
+	connections[hdl]=con;
 	}
 
-void Server::on_close(websocket_server::id_type ws_id)
+void Server::on_close(websocketpp::connection_hdl hdl)
 	{
 	std::lock_guard<std::mutex> lock(ws_mutex);
 	//	auto it = connections.find(hdl);
 	// snoop::log(snoop::info) << "Connection " << it->second.uuid << " close." << snoop::flush;
-	connections.erase(ws_id);
+	connections.erase(hdl);
 
 	if(exit_on_disconnect)
 		exit(-1);
@@ -327,33 +280,10 @@ void Server::on_close(websocket_server::id_type ws_id)
 
 int quit(void *)
 	{
-	std::cerr << "Server: setting python interrupt." << std::endl;
 	PyErr_SetInterrupt();
-	std::cerr << "Server: python interrupt set." << std::endl;
 	return -1;
 	}
 
-void Server::wait_for_websocket()
-	{
-	try {
-		wserver.set_message_handler(std::bind(&Server::on_message, this,
-														 std::placeholders::_1, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4));
-		wserver.set_connect_handler(std::bind(&Server::on_open, this,     std::placeholders::_1));
-		wserver.set_disconnect_handler(std::bind(&Server::on_close, this, std::placeholders::_1));
-
-		wserver.listen(run_on_port);
-
-		auto p = wserver.get_local_port();
-		std::cout << p  << std::endl;
-		std::cout << authentication_token << std::endl;
-		wserver.run();
-		}
-	catch(std::exception& ex) {
-		std::cerr << "Server::wait_for_websocket: exception " << ex.what() << std::endl;
-		throw;
-		}
-	}
-
 void Server::wait_for_job()
 	{
 	// Infinite loop, waiting for the master thread to signal that a new block is
@@ -385,7 +315,7 @@ void Server::wait_for_job()
 			block_available.wait(lock);
 			}
 
-		current_block = block_queue.front();
+		Block block = block_queue.front();
 		block_queue.pop();
 		lock.unlock();
 
@@ -397,11 +327,10 @@ void Server::wait_for_job()
 			// master thread can push new blocks onto it.
 			// snoop::log(snoop::info) << "Block finished running" << snoop::flush;
 			server_stopwatch.stop();
-			current_ws_id   = current_block.ws_id;
-			current_id      = current_block.cell_id;
-			current_block.output    = run_string(current_block.input, true, true, current_block.remove_variable_assignments);
-			current_block.variables = run_string_variables;
-			on_block_finished(current_block);
+			current_hdl=block.hdl;
+			current_id =block.cell_id;
+			block.output = run_string(block.input);
+			on_block_finished(block);
 			}
 		catch(std::runtime_error& ex) {
 #ifdef DEBUG
@@ -417,10 +346,8 @@ void Server::wait_for_job()
 			std::queue<Block> empty;
 			std::swap(block_queue, empty);
 			lock.unlock();
-			current_block.output = catchOut.str();
-			catchOut.clear();
-			current_block.error = ex.what();
-			on_block_error(current_block);
+			block.error = ex.what();
+			on_block_error(block);
 			}
 		catch(std::exception& ex) {
 			server_stopwatch.stop();
@@ -429,10 +356,8 @@ void Server::wait_for_job()
 			std::queue<Block> empty;
 			std::swap(block_queue, empty);
 			lock.unlock();
-			current_block.output=catchOut.str();
-			catchOut.clear();
-			current_block.error=ex.what();
-			on_kernel_fault(current_block);
+			block.error=ex.what();
+			on_kernel_fault(block);
 			// Keep running
 			}
 		}
@@ -440,28 +365,14 @@ void Server::wait_for_job()
 
 void Server::stop_block()
 	{
-//	interrupt_block=true;
-	std::cerr << "Server: sending SIGINT to python thread." << std::endl;
-	PyErr_SetInterrupt();
-
-	// PyGILState_STATE state = PyGILState_Ensure();
-	// //	PyThreadState_SetAsyncExc ?
-	// Py_AddPendingCall(&quit, NULL);
-	// PyGILState_Release(state);
-
-//	PyGILState_STATE state = PyGILState_Ensure();
-//	std::cerr << "Server: make thread " << main_thread_id << " raise exception" << std::endl;
-//	PyThreadState_SetAsyncExc(main_thread_id, PyExc_Exception);
-//	PyGILState_Release(state);
+	PyGILState_STATE state = PyGILState_Ensure();
+	//	PyThreadState_SetAsyncExc ?
+	Py_AddPendingCall(&quit, NULL);
+	PyGILState_Release(state);
 	}
 
-Server::Block::Block()
-	: ws_id(-1), cell_id(0)
-	{
-	}
-
-Server::Block::Block(websocket_server::id_type ws_id_, const std::string& str, uint64_t id_, const std::string& msg_type_)
-	: ws_id(ws_id_), msg_type(msg_type_), input(str), cell_id(id_)
+Server::Block::Block(websocketpp::connection_hdl h, const std::string& str, uint64_t id_, const std::string& msg_type_)
+	: hdl(h), msg_type(msg_type_), input(str), cell_id(id_)
 	{
 	nlohmann::json content, header;
 	response["header"]=header;
@@ -469,12 +380,11 @@ Server::Block::Block(websocket_server::id_type ws_id_, const std::string& str, u
 	response["msg_type"]=msg_type;
 	}
 
-void Server::on_message(websocket_server::id_type ws_id, const std::string& msg,
-								const websocket_server::request_type& req, const std::string& ip_address)
+void Server::on_message(websocketpp::connection_hdl hdl, WebsocketServer::message_ptr msg)
 	{
 	std::lock_guard<std::mutex> lock(ws_mutex);
 
-	auto it = connections.find(ws_id);
+	auto it = connections.find(hdl);
 	if(it==connections.end()) {
 //#ifndef ENABLE_JUPYTER
 //		snoop::log(snoop::warn) << "Message from unknown connection." << snoop::flush;
@@ -484,10 +394,10 @@ void Server::on_message(websocket_server::id_type ws_id, const std::string& msg,
 
 	//	std::cout << "Message from " << it->second.uuid << std::endl;
 
-	dispatch_message(ws_id, msg);
+	dispatch_message(hdl, msg->get_payload());
 	}
 
-void Server::dispatch_message(websocket_server::id_type ws_id, const std::string& json_msg)
+void Server::dispatch_message(websocketpp::connection_hdl hdl, const std::string& json_msg)
 	{
 	//	std::cout << json_msg << std::endl;
 
@@ -519,13 +429,7 @@ void Server::dispatch_message(websocket_server::id_type ws_id, const std::string
 		// std::cerr << code << std::endl;
 		uint64_t id = header.value("cell_id", uint64_t(0));
 		std::unique_lock<std::mutex> lock(block_available_mutex);
-		Block block(ws_id, code, id, msg_type);
-		if(content.count("remove_variable_assignments")==1) {
-			block.remove_variable_assignments.insert(content.value("remove_variable_assignments", ""));
-			}
-		if(header.count("output_cell_ids")==1) {
-			block.reuse_output_cell_ids = header.value("output_cell_ids", std::deque<uint64_t>());
-			}
+		Block block(hdl, code, id, msg_type);
 		block.response["header"]["parent_origin"]="client";
 		block.response["header"]["parent_id"]=id;
 		block_queue.push(block);
@@ -533,12 +437,10 @@ void Server::dispatch_message(websocket_server::id_type ws_id, const std::string
 		}
 	else if(msg_type=="execute_interrupt") {
 		std::unique_lock<std::mutex> lock(block_available_mutex);
-		std::cerr << "Server: requesting python thread stop." << std::endl;
 		stop_block();
-		std::cerr << "Server: clearing block queue." << std::endl;
+		//		std::cout << "clearing block queue" << std::endl;
 		std::queue<Block> empty;
 		std::swap(block_queue, empty);
-		std::cerr << "Server: block queue cleared." << std::endl;
 		//snoop::log(snoop::warn) << "Job stop requested." << snoop::flush;
 		}
 	else if(msg_type=="init") {
@@ -555,7 +457,7 @@ void Server::dispatch_message(websocket_server::id_type ws_id, const std::string
 		std::string todo="print(__cdbkernel__.completer.complete(\""+str+"\", "+std::to_string(alternative)+"))";
 
 		uint64_t id = header.value("cell_id", uint64_t(0));
-		Block blk(ws_id, todo, id, "completed");
+		Block blk(hdl, todo, id, "completed");
 		blk.response["header"]["cell_id"]=id;
 		blk.response["content"]["original"]=str;
 		blk.response["content"]["position"]=root["position"].get<int>();
@@ -590,14 +492,11 @@ void Server::on_block_finished(Block block)
 		header["cell_id"]=cadabra::generate_uuid<uint64_t>();
 		header["time_total_microseconds"]=std::to_string(server_stopwatch.seconds()*1e6L + server_stopwatch.useconds());
 		header["time_sympy_microseconds"]=std::to_string(sympy_stopwatch.seconds()*1e6L  + sympy_stopwatch.useconds());
-		header["last_in_sequence"]=block.error.empty(); // If this is the output followed by an error, it's not the last output cell for the running block.
+		header["last_in_sequence"]=true;
 		content["output"]=block.output;
 		block.response["msg_type"]="output";
 		}
 
-	// Inform the notebook about the variables referenced in this block.
-	content["variables"]=block.variables;
-
 	std::ostringstream str;
 	str << block.response << std::endl;
 	send_json(str.str());
@@ -605,33 +504,19 @@ void Server::on_block_finished(Block block)
 
 bool Server::handles(const std::string& otype) const
 	{
-	if(otype=="latex_view" || otype=="image_png" || otype=="image_svg" || otype=="verbatim") return true;
+	if(otype=="latex_view" || otype=="image_png" || otype=="verbatim") return true;
 	return false;
 	}
 
-uint64_t Server::send(const std::string& output, const std::string& msg_type,
-							 uint64_t parent_id, uint64_t cell_id, bool last)
+uint64_t Server::send(const std::string& output, const std::string& msg_type, uint64_t parent_id, bool last)
 	{
 	// This is the function exposed to the Python side; not used
 	// directly in the server to send block output back to the client
 	// (that's all handled by on_block_finished above).
 
-	// std::cerr << "Send: " << msg_type << ", " << output.substr(0, std::min(size_t(40), output.size())) << std::endl;
-	
 	nlohmann::json json, header, content;
 
-	uint64_t return_cell_id=cell_id;
-	if(return_cell_id==0) {
-		if(current_block.reuse_output_cell_ids.size()>0) {
-			// std::cerr << "Re-using existing output cell." << std::endl;
-			return_cell_id=current_block.reuse_output_cell_ids.front();
-			current_block.reuse_output_cell_ids.pop_front();
-			}
-		else {
-			return_cell_id=cadabra::generate_uuid<uint64_t>();
-			}
-		}
-	
+	auto return_cell_id=cadabra::generate_uuid<uint64_t>();
 	if(parent_id==0)
 		header["parent_id"]=current_id;
 	else
@@ -685,14 +570,11 @@ void Server::send_json(const std::string& msg)
 	{
 	//	std::cerr << "*** sending message " << msg << std::endl;
 	std::lock_guard<std::mutex> lock(ws_mutex);
-	wserver.send(current_ws_id, msg);
+	wserver.send(current_hdl, msg, websocketpp::frame::opcode::text);
 	}
 
 void Server::on_block_error(Block blk)
 	{
-	if(blk.output!="")
-		on_block_finished(blk);
-	
 	std::lock_guard<std::mutex> lock(ws_mutex);
 
 	// Make a JSON message.
@@ -714,14 +596,11 @@ void Server::on_block_error(Block blk)
 	str << json << std::endl;
 	// std::cerr << "cadabra-server: sending error, " << str.str() << std::endl;
 
-	wserver.send(blk.ws_id, str.str());
+	wserver.send(blk.hdl, str.str(), websocketpp::frame::opcode::text);
 	}
 
 void Server::on_kernel_fault(Block blk)
 	{
-	if(blk.output!="")
-		on_block_finished(blk);
-	
 	std::lock_guard<std::mutex> lock(ws_mutex);
 
 	// Make a JSON message.
@@ -743,25 +622,44 @@ void Server::on_kernel_fault(Block blk)
 	str << json << std::endl;
 	// std::cerr << "cadabra-server: sending kernel crash report, " << str.str() << std::endl;
 
-	wserver.send(blk.ws_id, str.str());
+	wserver.send(blk.hdl, str.str(), websocketpp::frame::opcode::text);
 	}
 
 void Server::run(int port, bool eod)
 	{
-	exit_on_disconnect = eod;
-	run_on_port = port;
+	exit_on_disconnect=eod;
+	try {
+		wserver.clear_access_channels(websocketpp::log::alevel::all);
+		wserver.clear_error_channels(websocketpp::log::elevel::all);
 
-	// Python has to be running on the main thread, otherwise
-	// it cannot receive signals. So we spawn the websocket
-	// listener on a separate thread, and then do the blocking
-	// "wait for python code to execute" loop on the main
-	// thread.
+		wserver.init_asio();
+		wserver.set_reuse_addr(true);
 
-//	std::thread::id tmp= std::this_thread::get_id();	
-//	main_thread_id = *(unsigned *)&tmp;
-//	std::cerr << "Server: main_thread_id = " << main_thread_id << std::endl;
-	runner = std::thread(std::bind(&Server::wait_for_websocket, this));
+		wserver.set_socket_init_handler(bind(&Server::on_socket_init, this, ::_1,::_2));
+		wserver.set_message_handler(bind(&Server::on_message, this, ::_1, ::_2));
+		wserver.set_open_handler(bind(&Server::on_open,this,::_1));
+		wserver.set_close_handler(bind(&Server::on_close,this,::_1));
 
-	wait_for_job();
-//		pybind11::gil_scoped_release release;
+#ifdef DEBUG
+		std::cerr << "going to listen" << std::endl;
+#endif
+		wserver.listen(websocketpp::lib::asio::ip::tcp::v4(), port);
+#ifdef DEBUG
+		std::cerr << "going to accept" << std::endl;
+#endif
+		wserver.start_accept();
+		websocketpp::lib::asio::error_code ec;
+		auto p = wserver.get_local_endpoint(ec);
+		std::cout << p.port()  << std::endl;
+		std::cout << authentication_token << std::endl;
+
+		// std::cerr << "cadabra-server: spawning job thread "  << std::endl;
+		runner = std::thread(std::bind(&Server::wait_for_job, this));
+
+		pybind11::gil_scoped_release release;
+		wserver.run();
+		}
+	catch(websocketpp::exception& ex) {
+		std::cerr << "cadabra-server: websocket exception " << ex.code() << " " << ex.what() << std::endl;
+		}
 	}
