@@ -37,10 +37,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "properties/Symbol.hh"
 #include "properties/Trace.hh"
 #include "properties/DependsBase.hh"
-#include "properties/NonCommuting.hh"
-#include "properties/AntiCommuting.hh"
-#include "properties/SelfNonCommuting.hh"
-#include "properties/SelfAntiCommuting.hh"
 
 #include <sstream>
 
@@ -880,23 +876,6 @@ bool Algorithm::is_nonprod_factor_in_prod(iterator it)
 	return false;
 	}
 
-bool Algorithm::is_noncommuting(const Properties& properties, iterator it)
-	{
-	auto nc = properties.get<NonCommuting>(it);
-	if(nc) return true;
-
-	auto ac = properties.get<AntiCommuting>(it);
-	if(ac) return true;
-	
-	auto snc = properties.get<SelfNonCommuting>(it);
-	if(snc) return true;
-	
-	auto sac = properties.get<SelfAntiCommuting>(it);
-	if(sac) return true;
-
-	return false;
-	}
-
 bool Algorithm::prod_wrap_single_term(iterator& it)
 	{
 	if(is_single_term(it)) {
@@ -1081,7 +1060,7 @@ bool Algorithm::locate_object_set(const Ex& objs,
 		if((*aim->name)=="\\comma") {
 			// Objects can themselves be lists of other objects (for instance
 			// when we want to symmetrise in index pairs).
-			if(locate_object_set(Ex(aim), st, nd, store)==false)
+			if(locate_object_set(aim, st, nd, store)==false)
 				return false;
 			}
 		else {
