@@ -10,10 +10,6 @@
 #include "properties/Tableau.hh"
 #include "properties/FilledTableau.hh"
 #include "properties/TableauInherit.hh"
-#include "NTensor.hh"
-#include "NInterpolatingFunction.hh"
-
-#include <regex>
 
 #define nbsp   " "
 //(( parent.utf8_output?(unichar(0x00a0)):" "))
@@ -336,22 +332,13 @@ void DisplayTeX::print_children(std::ostream& str, Ex::iterator it, int skip)
 			}
 
 		if(current_bracket_!=str_node::b_none || previous_bracket_!=current_bracket_ || previous_parent_rel_!=current_parent_rel_) {
-			if(previous_parent_rel_!=current_parent_rel_) 
-				if(previous_parent_rel_==str_node::p_sub || previous_parent_rel_==str_node::p_super)
-					str << "\\!";
-			
 			print_parent_rel(str, current_parent_rel_, ch==tree.begin(it));
 
-			if(is_accent==0 && function_bracket_needed) {
-				// FIXME: the line below was there originally to make function brackets sit a bit
-				// closer to the function name. However, it gets triggered for
-				//   `\alpha_{n}^{\mu}::SelfNonCommuting;`
-				// and then messes up the superscript. That's worse than wrong spacing.
+			if(is_accent==0 && function_bracket_needed)
 				print_opening_bracket(str, (number_of_nonindex_children>1 /* &&number_of_index_children>0 */ &&
 				                            current_parent_rel_!=str_node::p_sub &&
 				                            current_parent_rel_!=str_node::p_super ? str_node::b_round:current_bracket_),
 				                      current_parent_rel_);
-				}
 			else str << "{";
 			}
 
@@ -377,33 +364,21 @@ void DisplayTeX::print_children(std::ostream& str, Ex::iterator it, int skip)
 
 void DisplayTeX::print_multiplier(std::ostream& str, Ex::iterator it, int mult)
 	{
-	if(it->multiplier->is_rational()) {
-		mpq_class q = it->multiplier->get_rational();
-		mpz_class denom=q.get_den();
-		mpz_class numer=q.get_num();
-		
-		if(denom!=1) {
-			if(mult*numer<0) {
-				str << " - ";
-				mult *= -1;
-				}
-			str << "\\frac{" << mult * numer << "}{" << denom << "}";
-			}
-		else if(mult * q==-1) {
-			str << "-";
-			}
-		else {
-			str << mult * q;
+	mpz_class denom=it->multiplier->get_den();
+
+	if(denom!=1) {
+		if(mult*it->multiplier->get_num()<0) {
+			str << " - ";
+			mult *= -1;
 			}
+		str << "\\frac{" << mult * it->multiplier->get_num() << "}{" << it->multiplier->get_den() << "}";
+		}
+	else if(mult * (*it->multiplier)==-1) {
+		str << "-";
 		}
 	else {
-		std::ostringstream str2;
-		str2 << it->multiplier->get_double();
-		std::string s = str2.str();
-		std::string s2 = std::regex_replace(s, std::regex("(\\d+\\.?\\d*|\\.\\d+)e([+-]?\\d+)"), "$1 \\times 10^{$2}");
-		str << s2;
+		str << mult * (*it->multiplier);
 		}
-	str << "\\,";
 	}
 
 void DisplayTeX::print_opening_bracket(std::ostream& str, str_node::bracket_t br, str_node::parent_rel_t pr)
@@ -486,37 +461,6 @@ void DisplayTeX::print_parent_rel(std::ostream& str, str_node::parent_rel_t pr,
 
 void DisplayTeX::dispatch(std::ostream& str, Ex::iterator it)
 	{
-	if(handle_unprintable_wildcards(str, it))
-		return;
-
-	if(std::holds_alternative<std::shared_ptr<NTensor>>(it->content)) {
-		std::ostringstream str2;
-		auto nt = std::get<std::shared_ptr<NTensor>>(it->content);
-		str2 << *nt;
-		std::string s = str2.str();
-		std::string s2 = std::regex_replace(s, std::regex("(\\d+\\.?\\d*|\\.\\d+)e([+-]?\\d+)"), "$1 \\times 10^{$2}");
-		if(nt->shape.size()==1 && nt->shape[0]==1) {
-			if(!nt->is_real())
-				str << "(" << s2.substr(1, s2.size()-2) << ")";
-			else
-				str << s2.substr(1, s2.size()-2);
-			}
-		else
-			str << s2;
-
-		if(*it->name=="1" && tree.number_of_children(it)==0)
-			return;
-//		return; // FIXME: always just ignore the subtree?
-		}
-	else if(std::holds_alternative<std::shared_ptr<NInterpolatingFunction>>(it->content)) {
-		str << "\\square{}" << std::endl;
-		auto ni = std::get<std::shared_ptr<NInterpolatingFunction>>(it->content);
-		str << "(";
-		dispatch(str, ni->var.begin());
-		str << ")";
-		return;
-		}
-	
 	if(*it->name=="\\prod")                                   print_productlike(str, it, " ");
 	else if(*it->name=="\\sum" || *it->name=="\\oplus")       print_sumlike(str, it);
 	else if(*it->name=="\\frac")                              print_fraclike(str, it);
@@ -548,7 +492,7 @@ void DisplayTeX::print_commalike(std::ostream& str, Ex::iterator it)
 		if(first)
 			first=false;
 		else
-			str << ", " << discr; // << " ";
+			str << ",~" << discr << " ";
 		dispatch(str, sib);
 		++sib;
 		}
@@ -605,18 +549,11 @@ void DisplayTeX::print_fraclike(std::ostream& str, Ex::iterator it)
 		str << " - ";
 		mult=-1;
 		}
-
 	str << "\\frac{";
-	if(it->multiplier->is_rational()) {
-		if(mult * (it->multiplier->get_rational())!=1) {
-			print_multiplier(str, it, mult);
-			}
-		}
-	else {
+	if(mult * (*it->multiplier)!=1) {
 		print_multiplier(str, it, mult);
 		}
-	
-	if(num->is_rational()==false || (mult * (it->multiplier->get_rational()))==1)
+	if(num->is_rational()==false || (mult * (*it->multiplier))==1)
 		dispatch(str, num);
 	str << "}{";
 	dispatch(str, den);
@@ -661,20 +598,13 @@ void DisplayTeX::print_productlike(std::ostream& str, Ex::iterator it, const std
 				str << "-";
 				mult *= -1;
 			}
-			if (mult.is_rational()) {
-				auto rmult = mult.get_rational();
-				if(mult.get_rational().get_den() == 1) {
-					multiply(pos.begin()->multiplier, rmult);
-					}
-				else {
-					multiply(pos.begin()->multiplier, rmult.get_num());
-					multiply(neg.begin()->multiplier, rmult.get_den());
-					}
-				}
-			else {
+			if (mult.get_den() == 1) {
 				multiply(pos.begin()->multiplier, mult);
-				}
-			
+			}
+			else {
+				multiply(pos.begin()->multiplier, mult.get_num());
+				multiply(neg.begin()->multiplier, mult.get_den());
+			}
 			str << "\\frac{";
 			if (pos.begin().begin() == pos.begin().end()) {
 				pos.begin()->name = name_set.insert("1").first;
@@ -777,10 +707,8 @@ void DisplayTeX::print_sumlike(std::ostream& str, Ex::iterator it)
 	while(ch!=tree.end(it)) {
 		//		if(ch!=tree.begin(it))
 		//			str << "%\n"; // prevent LaTeX overflow.
-		if(steps>0)
-			str << discr;
 		if(++steps==20) {
-			steps=1;
+			steps=0;
 			str << "%\n"; // prevent LaTeX overflow.
 			}
 		if(*ch->multiplier>=0 && ch!=tree.begin(it)) {
@@ -810,33 +738,17 @@ void DisplayTeX::print_sumlike(std::ostream& str, Ex::iterator it)
 	str << std::flush;
 	}
 
-bool DisplayTeX::handle_unprintable_wildcards(std::ostream& str, Ex::iterator it) const
-	{
-	// Catch `\pow{#}` and other wildcard constructions, these
-	// need to print verbatim.
-
-	if(it.number_of_children()==1) {
-		const std::string& name = (*it->name);
-		if(name.size()>0 && name[0]=='\\' && *(it.begin()->name)=="#") {
-			str << "\\backslash\\texttt{" << name.substr(1) << "}\\{\\#\\}";
-			return true;
-			}
-		}
-	return false;
-	}
-
 void DisplayTeX::print_powlike(std::ostream& str, Ex::iterator it)
 	{
 	auto arg=tree.begin(it);
 	assert(arg!=tree.end(it));
 	auto exp=arg;
 	++exp;
-
 	assert(exp!=tree.end(it));
 
 	if (kernel.display_fractions && exp->is_rational() && *exp->multiplier < 0) {
 		auto mult = *it->multiplier;
-		bool mult_is_int = mult.is_rational() && (mult.get_rational().get_den() == 1);
+		bool mult_is_int = mult.get_den() == 1;
 		if (mult < 0) {
 			str << "-";
 			mult *= -1;
@@ -844,22 +756,13 @@ void DisplayTeX::print_powlike(std::ostream& str, Ex::iterator it)
 		str << "\\frac{";
 		if (mult_is_int)
 			str << mult;
-		else {
-			if(mult.is_rational())
-				str << mult.get_rational().get_num();
-			else
-				str << mult;
-			}
+		else
+			str << mult.get_num();
 		str << "}{";
 		if (*exp->multiplier == -1) {
 			Ex copy(arg);
-			if(mult.is_rational()) {
-				if (!mult_is_int) {
-					multiply(copy.begin()->multiplier, mult.get_rational().get_den());
-					}
-				}
-			else multiply(copy.begin()->multiplier, mult);
-			
+			if (!mult_is_int)
+				multiply(copy.begin()->multiplier, mult.get_den());
 			dispatch(str, copy.begin());
 		}
 		else {
@@ -867,12 +770,8 @@ void DisplayTeX::print_powlike(std::ostream& str, Ex::iterator it)
 			exp = copy.begin().begin();
 			++exp;
 			multiply(exp->multiplier, -1);
-			if(mult.is_rational()) {
-				if (!mult_is_int)
-					copy.begin()->multiplier = rat_set.insert(mult.get_rational().get_den()).first;
-				}
-			else copy.begin()->multiplier = rat_set.insert(mult).first;
-			
+			if (!mult_is_int)
+				copy.begin()->multiplier = rat_set.insert(mult.get_den()).first;
 			print_powlike(str, copy.begin());
 		}
 		str << "}";
